function optionsAddEventListener () {
    var options = document.querySelectorAll ("#options input, #options select"),
        notice = document.querySelector ("#notice span");
        
    for (var i = 0; i < options.length; i++) {
        options[i].dataset.saved = true;
        
        options[i].addEventListener("change", function () {
            var self = this;
            if (self.dataset.saved == "false" || self.type == "select-one" || self.type == "checkbox" || self.type == "radio") {
                optionsSave (self);
                self.dataset.saved = "true";
            }
        }, false);
        
        var timer = 0;
        options[i].addEventListener("keyup", function () {
            var self = this;
            
            if (notice.innerHTML != "Saving...") {
                uiUpdateNotice ("Saving...");
            }
            
            clearTimeout (timer);
            
            if (self.dataset.saved == "true") {
                self.dataset.saved = "false";
            }
            
            timer = setTimeout (function () {
                if (self.dataset.saved == "false") {
                    optionsSave (self);
                    self.dataset.saved = true;
                }
            }, 2000);
        }, false);
    }
}
function optionsRestore () {
    var options = document.querySelectorAll ("#options input, #options select");
    
    for (var i = 0; i < options.length; i++) {
        var self = options[i],
            tempValue = localStorage.getItem(self.id, self.value);
        
        if (self.type == "select-one") {
            if (tempValue == null) {
                self.options.selectedIndex = 0;
            } else {
                self.options.selectedIndex = self.options.namedItem(tempValue).index;
            }
        } else if (self.type == "checkbox" || self.type == "radio") {
            if (tempValue == "true" || tempValue == null) {
                self.setAttribute("checked", "checked");
            } else {
                self.removeAttribute("checked");
            }
        } else {
            self.value = tempValue;
        }
    }
}
function optionsSave (self) {
    uiUpdateNotice ("Saved");
    localStorage.setItem(self.id, self.value);
    console.log (self.id + " = " + self.value);
}

var ui = wright.design ({
    "attribs": {
        "id": "wrapper"
    },
    "content": [
        {
            "attribs": {
                "id": "title"
            },
            "content": {
                "type": "span",
                "content": "Options"
            }
        },
        {
            "attribs": {
                "id": "notice"
            },
            "content": {
                "type": "span",
                "content": "Options are immediately saved after entering them"
            }
        },
        {
            "attribs": {
                "id": "content"
            },
            "content": {
                "attribs": {
                    "id": "options",
                    "class": ["list", "smooth"]
                },
                "content": [
                    {
                        "attribs": {
                            "class": ["option"]
                        },
                        "content": [
                            {
                                "attribs": {
                                    "class": ["info"]
                                },
                                "content": [
                                    {
                                        "type": "label",
                                        "content": "Provider"
                                    },
                                    {
                                        "type": "span",
                                        "content": "The back-end provider for streaming."
                                    }
                                ]
                            },
                            {
                                "attribs": {
                                    "class": ["setting"]
                                },
                                "content": {
                                    "type": "select",
                                    "attribs": {
                                        "id": "provider"
                                    },
                                    "content": [
                                        {
                                            "type": "option",
                                            "attribs": {
                                                "id": "livestreamer",
                                                "value": "livestreamer",
                                                "selected": "selected"
                                            },
                                            "content": "livestreamer"
                                        },
                                        {
                                            "type": "option",
                                            "attribs": {
                                                "id": "yotube-dl",
                                                "value": "yotube-dl"
                                            },
                                            "content": "yotube-dl"
                                        }
                                    ]
                                }
                            }
                        ]
                    },
                    {
                        "attribs": {
                            "class": ["option"]
                        },
                        "content": [
                            {
                                "attribs": {
                                    "class": ["info"]
                                },
                                "content": [
                                    {
                                        "type": "label",
                                        "content": "Path"
                                    },
                                    {
                                        "type": "span",
                                        "content": "Path to livestreamer executable, leave blank to detect automatically."
                                    }
                                ]
                            },
                            {
                                "attribs": {
                                    "class": ["setting"]
                                },
                                "content": {
                                    "type": "input",
                                    "attribs": {
                                        "id": "path",
                                        "type": "text"
                                    },
                                    "content": ""
                                }
                            }
                        ]
                    },
                    {
                        "attribs": {
                            "class": ["option"]
                        },
                        "content": [
                            {
                                "attribs": {
                                    "class": ["info"]
                                },
                                "content": [
                                    {
                                        "type": "label",
                                        "content": "Optional Arguments"
                                    },
                                    {
                                        "type": "span",
                                        "content": "You can pass livestreamer options here in the form of <code>--foo=bar,--baz=foo</code>. Check <code>livestreamer -h</code> for what is available."
                                    }
                                ]
                            },
                            {
                                "attribs": {
                                    "class": ["setting"]
                                },
                                "content": {
                                    "type": "input",
                                    "attribs": {
                                        "id": "optargs",
                                        "type": "text"
                                    },
                                    "content": ""
                                }
                            }
                        ]
                    },
                    {
                        "attribs": {
                            "class": ["option"]
                        },
                        "content": [
                            {
                                "attribs": {
                                    "class": ["info"]
                                },
                                "content": [
                                    {
                                        "type": "label",
                                        "content": "Quality Menu"
                                    },
                                    {
                                        "type": "span",
                                        "content": "Display a quality selection window everytime livestreamer is launched."
                                    }
                                ]
                            },
                            {
                                "attribs": {
                                    "class": ["setting"]
                                },
                                "content": {
                                    "type": "input",
                                    "attribs": {
                                        "id": "menu",
                                        "type": "checkbox",
                                        "checked": "checked",
                                        "value": "true"
                                    }
                                }
                            }
                        ]
                    },
                    {
                        "attribs": {
                            "class": ["option"]
                        },
                        "content": [
                            {
                                "attribs": {
                                    "class": ["info"]
                                },
                                "content": [
                                    {
                                        "type": "label",
                                        "content": "Stream Quality"
                                    },
                                    {
                                        "type": "span",
                                        "content": "Change the quality of the stream, options are <code>low</code>, <code>medium</code>, <code>best</code>. Only applied if quality menu is disabled."
                                    }
                                ]
                            },
                            {
                                "attribs": {
                                    "class": ["setting"]
                                },
                                "content": {
                                    "type": "input",
                                    "attribs": {
                                        "id": "quality",
                                        "type": "text"
                                    },
                                    "content": ""
                                }
                            }
                        ]
                    },
                    {
                        "attribs": {
                            "class": ["option"]
                        },
                        "content": [
                            {
                                "attribs": {
                                    "class": ["info"]
                                },
                                "content": [
                                    {
                                        "type": "label",
                                        "content": "Context Menu"
                                    },
                                    {
                                        "type": "span",
                                        "content": "Display a context menu when you right click a page or link with options to launch a stream."
                                    }
                                ]
                            },
                            {
                                "attribs": {
                                    "class": ["setting"]
                                },
                                "content": {
                                    "type": "input",
                                    "attribs": {
                                        "id": "contextmenu",
                                        "type": "checkbox",
                                        "checked": "checked",
                                        "value": "true"
                                    }
                                }
                            }
                        ]
                    }
                ]
            }
        }
    ]
});

if (document.body != null) {
    document.body.appendChild(ui);
}

var options = document.querySelectorAll ("#options input, #options select"),
    notice = document.querySelector ("#notice span");
    
optionsRestore ();
optionsAddEventListener ();